/*
 * This file is part of Goal Based City Growth (GBCG), which is a GameScript for OpenTTD
 * Copyright (C) 2016 Patrick Brennan
 *
 * GBCG is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * GBCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GBCG; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA 02110-1301 USA.
 */

enum CatLabels
{
	PAXMAIL = 0,
	G_FOOD = 1,
	G_GOODS = 2,
	RAW_IND = 3,
	TR_IND = 4,
	G_IND = 5
}

//***************************************************************************
// Cargolist of supported industry set's cargos. Used to check if
// initial ingame cargo list matches the defined setting. Needs to be
// updated if industry set cargos change.
//***************************************************************************
function FillCargoIDList() {
	switch (GSController.GetSetting("industry_NewGRF")) {
	/* Baseset */
	case(1): // Temperate
		::CargoIDList <- ["PASS","COAL","MAIL","OIL_","LVST","GOOD",
				  "GRAI","WOOD","IORE","STEL","VALU"];
		break;
	case(2): // Arctic
		::CargoIDList <- ["PASS","COAL","MAIL","OIL_","LVST","GOOD",
				  "WHEA","WOOD",null,"PAPR","GOLD","FOOD"];
		break;
	case(3): // Tropical
		::CargoIDList <- ["PASS","RUBR","MAIL","OIL_","FRUT","GOOD",
				  "MAIZ","WOOD","CORE","WATR","DIAM","FOOD"];
		break;
	/* FIRS version 2.1.5 */
	case(4): // Temperate basic
		::CargoIDList <- ["PASS","BEER","MAIL","RFPR","CLAY","GOOD","COAL","ENSP","FMSP","FISH",
				  "FRUT","FOOD","IORE","LVST","MNSP","STEL","MILK","SAND","SCMT"];
		break;
	case(5): // Arctic basic
		::CargoIDList <- ["PASS","AORE","MAIL","RFPR","CLAY","GOOD","ENSP","FMSP","STEL","OIL_",
				  "PAPR","FOOD","PETR","PORE","RUBR","SAND","SCMT","VPTS","VEHI","WOOD"];
		break;
	case(6): // Tropic basic
		::CargoIDList <- ["PASS","BEER","MAIL","BEAN","RFPR","GOOD","JAVA","COPR","CORE","ENSP",
				  "FMSP","FOOD","FISH","FRUT","GRAI","LVST","MNSP","NITR","OIL_","WOOL"];
		break;
	case(7): // In A Hot Country
		::CargoIDList <- ["PASS","BEER","MAIL","BDMT","CASS","GOOD","RFPR","CLAY","JAVA","COPR",
				  "CORE","FOOD","DIAM","EOIL","ENSP","FMSP","FRUT","LVST","WDPR","MAIZ",
				  "MNO2","MNSP","NUTS","OIL_","PETR","PHOS","RUBR","SAND","GRVL","WOOD"];
		break;
	case(8): // Extreme
		::CargoIDList <- ["PASS","BEER","MAIL","AORE","BDMT","GOOD","RFPR","CLAY","COAL","ENSP",
				  "FMSP","FOOD","FISH","FRUT","GRAI","IORE","LVST","WDPR","MNSP","STEL",
				  "MILK","OIL_","PETR","FICR","RCYC","SAND","SCMT","GRVL","SGBT","WOOD",
				  "WOOL"];
		break;
	case(9): // Mars
		::CargoIDList <- ["PASS",null,"N2__","O2__","H2__","CHEM","ORES","ORGN","FERT","PLAS","BDMT",
				  "FOOD","GOOD","ENSP","WATR","GRVL"];//15
		break;
	default: break;
	}

	// This is used to add all null IDs after the end of active used cargos
	local missing_cargo = 32 - ::CargoIDList.len();
	for (local i = 0; i < missing_cargo; i++) {
		for (local i = 0; i < missing_cargo; i++) ::CargoIDList.append(null);
	}
}

//***************************************************************************
// Here are defined the cargo categories and other data for each
// set. Follow explanations below to change category data.
//***************************************************************************
function DefineCargosBySettings()
{
	/* Setup global cargo variables, per cargo categories: (Cat 1:
	 * Passengers and mail, Cat 2: General food, Cat 3: General
	 * goods, Cat 4: Raw industrial materials, Cat 5: transformed
	 * indudstrial goods). For base sets, only 3 categories are
	 * defined (PaxMail, General goods (including food),
	 * industrial goods).
	 *
	 * For each industry set, we define:
	 * ::CargoCat: the base categories of cargo. For each
	 * category, an array of cargo types is created. For cargo Id
	 * references, enable debugging and watch at the AI/GameScript
	 * log.
	 * ::CargoCatList: a list of used cargo categories; see enum
	 * CargoCategories
	 * ::CargoMinPopDemand: minimal town population at which a
	 * cargo category is required
	 * ::CargoPermille: a factor defining the growth of cargo category
	 * requirements
	 * ::CargoDecay: defines the part of the stockpiled cargos
	 * which is lost each month
	 */
	local industry_setting = GSController.GetSetting("industry_NewGRF");
	switch (industry_setting) {
		case(1): // Base temperate
			::CargoCat <- [[0,2],
				       [5,10],
				       [1,3,4,6,7,8,9]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_GOODS,CatLabels.G_IND];
			::CargoMinPopDemand <- [0,1000,4000];
			::CargoPermille <- [60,45,25];
			::CargoDecay <- [0.4,0.2,0.1];
			break;
		case(2): //Base arctic
			::CargoCat <- [[0,2],
				       [5,10,11],
				       [1,3,4,6,7,9]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_GOODS,CatLabels.G_IND];
			::CargoMinPopDemand <- [0,1000,4000];
			::CargoPermille <- [60,45,25];
			::CargoDecay <- [0.4,0.2,0.1];
			break;
		case(3): //Base tropical
			::CargoCat <- [[0,2],
				       [5,9,10,11],
				       [1,3,4,6,7,8]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_GOODS,CatLabels.G_IND];
			::CargoMinPopDemand <- [0,1000,4000];
			::CargoPermille <- [60,45,25];
			::CargoDecay <- [0.4,0.2,0.1];
			break;
		case(4): // FIRS - Temperate Basic
			::CargoCat <- [[0,2],
				       [9,10,11,13,16],
				       [1,5],
				       [4,6,12,17,18],
				       [3,7,8,14,15]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_FOOD,CatLabels.G_GOODS,
					   CatLabels.RAW_IND,CatLabels.TR_IND];
			::CargoMinPopDemand <- [0,500,1000,4000,8000];
			::CargoPermille <- [60,25,20,15,10];
			::CargoDecay <- [0.4,0.2,0.2,0.1,0.1];
			break;

		case(5): // FIRS - Arctic Basic
			::CargoCat <- [[0,2],
				       [11],
				       [5,10,12,14],
				       [4,9,13,15,16,19],
				       [1,3,6,7,8,17,18]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_FOOD,CatLabels.G_GOODS,
					   CatLabels.RAW_IND,CatLabels.TR_IND];
			::CargoMinPopDemand <- [0,500,1000,4000,8000];
			::CargoPermille <- [60,25,20,15,10];
			::CargoDecay <- [0.4,0.2,0.2,0.1,0.1];
			break;
		case(6): // FIRS - Tropic Basic
			::CargoCat <- [[0,2],
				       [1,3,6,11,12,13,14,15],
				       [5],
				       [4,8,17,18,19],
				       [7,9,10,16]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_FOOD,CatLabels.G_GOODS,
					   CatLabels.RAW_IND,CatLabels.TR_IND];
			::CargoMinPopDemand <- [0,500,1000,4000,8000];
			::CargoPermille <- [60,25,20,15,10];
			::CargoDecay <- [0.4,0.2,0.2,0.1,0.1];
			break;
		case(7): // FIRS - In A Hot Country
			::CargoCat <- [[0,2],
				       [4,8,11,16,17,19,22],
				       [1,3,5,24,26],
				       [7,10,12,20,23,25,27,28,29],
				       [6,9,13,14,15,18,21]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_FOOD,CatLabels.G_GOODS,
					   CatLabels.RAW_IND,CatLabels.TR_IND];
			::CargoMinPopDemand <- [0,500,1000,4000,8000];
			::CargoPermille <- [60,25,20,15,10];
			::CargoDecay <- [0.4,0.2,0.2,0.1,0.1];
			break;
		case(8): // FIRS - Extreme
			::CargoCat <- [[0,2],
				       [11,12,13,14,16,20,23,28],
				       [1,4,5,22],
				       [7,8,15,21,24,25,26,27,29,30],
				       [3,6,9,10,17,18,19]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_FOOD,CatLabels.G_GOODS,
					   CatLabels.RAW_IND,CatLabels.TR_IND];
			::CargoMinPopDemand <- [0,500,1000,4000,8000];
			::CargoPermille <- [60,25,20,15,10];
			::CargoDecay <- [0.4,0.2,0.2,0.1,0.1];
			break;
		case(9): // mars
			::CargoCat <- [[0],
				       [11,14,3],
				       [12],
					   [4],
					   [10,13,5]];
			::CargoCatList <- [CatLabels.PAXMAIL,CatLabels.G_FOOD,CatLabels.G_GOODS,CatLabels.RAW_IND,CatLabels.TR_IND];
			::CargoMinPopDemand <- [0,500,9000,15000,28000];
			::CargoPermille <- [30,90,20,15,10];
			::CargoDecay <- [0.4,0.2,0.1,0.1,0.1];
			break;
		default:
			break;
			
			
	}
}

//***************************************************************************
// This function compares the ingame initial cargo list to the
// industry sets and cargoscheme supported by the script. Return false
// if they don't match.
//***************************************************************************

function CheckInitialCargoList() {
	local cargo_list = [];
	Log.Info("Detecting cargo..", Log.LVL_INFO);
	for(local i = 0; i < 32; ++i) {
		cargo_list.append(GSCargo.GetCargoLabel(i));
		Log.Info("Cargo detected ["+i+"]: " + GSCargo.GetCargoLabel(i), Log.LVL_DEBUG);
	}
	FillCargoIDList();

	// Comparing actual list to original list
	for (local i = 0; i < 32; i++) {
		
		if (cargo_list[i] != ::CargoIDList[i]) {
			::CargoIDList = false;

			Log.Info("Warning: Game's cargo list differ from settings.", Log.LVL_INFO);
			return;
		}
	}
}

//***************************************************************************
// Initialization of cargo data. 
//***************************************************************************
function InitCargoLists()
{
	DebugCargoLabels();       // Debug info: print cargo labels

	CheckInitialCargoList()   // Check if cargo list matches the settings

	DefineCargosBySettings(); // Define cargo data accordingly to industry set

	// Building a general list of all cargos, ordered by categories.
	::CargoList <- [];
	for (local i = 0; i < ::CargoCat.len(); i++) {
		for (local j = 0; j < ::CargoCat[i].len(); j++) {
			::CargoList.append(CargoCat[i][j]);
		}
	}

	// Initializing some useful and often used variables
	::CargoTypeNum <- ::CargoList.len();
	::CargoCatNum <- ::CargoCat.len();
}


//***************************************************************************
// Goals Cargo description.
//***************************************************************************
class Cargo
{
    index   = null; ///< Index of the cargo in the cargo table (#MainClass.cargoes).
    cid     = null; ///< Id of the cargo in GRF.
    freight = null; ///< Whether the cargo is considered to be freight.
    effect  = null; ///< Town effect (One of #GSCargo.TownEffect).
    weight  = null; ///< Likelihood of picking this cargo for making a goal.

    constructor(index, cid, freight, effect)
    {
        this.index = index;
        this.cid = cid;
        this.freight = freight;
        this.effect = effect;
        this.weight = this.GetWeight(effect);
    }

    function GetWeight();
}
//***************************************************************************
// Get the weight of the cargo (probability of selecting it).
// @param effect Town effect of the cargo.
//***************************************************************************
function Cargo::GetWeight(effect)
{
    if (effect == GSCargo.TE_PASSENGERS) return GSController.GetSetting("pass_weight");
    if (effect == GSCargo.TE_MAIL)       return GSController.GetSetting("mail_weight");
    if (effect != GSCargo.TE_NONE)       return GSController.GetSetting("town_weight");
    return 1;
}

