/*
 * This file is part of Goal Based City Growth (GBCG), which is a GameScript for OpenTTD
 * Copyright (C) 2016 Patrick Brennan
 *
 * GBCG is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * GBCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GBCG; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA 02110-1301 USA.
 */

class CompanyData {
    comp_id = null;
    active_goals = null;

    constructor(comp_id) {
        this.active_goals = {};
        this.comp_id = comp_id;
        for (local num = 0; num < 32; num += 1) this.active_goals[num] <- null;
	}
	
    function AddActiveGoal(cargo, accept, amount, year_expiry, goal_type);
    function HasGoal(cargo_id, accept);
    function HasGrowthGoal(town_id);
    function HasPickupGoal(type);
    function GetNumberOfGoals(scope);
    function GetNumberOfGoalsForCargo(cargo_id);
    function UpdateGoalsProgress();
    function IndustryClosed(ind_id);
	function StartMonitoringPickupGoals();
    function CheckAndFinishGoals();
    function ExpireGoals();
    function FinalizeCompany();
    function SaveCompany();
    function GoalsPostLoadCheck();
    static function LoadCompany(comp_id, loaded_data);
};

//***************************************************************************
// Add a goal to the list of the company.
// @param cargo Cargo of the goal (#Cargo).
// @param accept Accepting resource of the goal.
// @param wanted_amount Amount of cargo to deliver.
//***************************************************************************
function CompanyData::AddActiveGoal(cargo, accept, wanted_amount, year_expiry, goal_type, goal_scope)
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) {
            this.active_goals[num] = CompanyGoal(this.comp_id, cargo, accept, wanted_amount, year_expiry, goal_type, goal_scope);
			break;
        }
    }
}

//***************************************************************************
// @return Number of active goals
// @param scope: if null, return total number of active goals; otherwise return number of active goals for the specified scope
//***************************************************************************
function CompanyData::GetNumberOfGoals(scope)
{
    local count = 0;
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
		if (scope == null) {
			count += 1;
		} else {
			if (goal.goal_scope == scope) {
				count += 1;
			}
		}
    }
    return count;
}

//***************************************************************************
// Does the company have an active goal for the given cargo and accepting resource?
// @param cargo_id Cargo to check.
// @param accept Accepting resource to check.
// @return Whether the company has a goal for the cargo and resource.
//***************************************************************************
function CompanyData::HasGoal(cargo_id, accept)
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
        if (goal.cargo.cid != cargo_id) continue;
		if ("town" in accept) {
            if ("ind" in goal.accept || "type" in goal.accept || accept.town != goal.accept.town) continue;
		} else {
			if ("ind" in accept) {
				if ("town" in goal.accept || "type" in goal.accept || accept.ind != goal.accept.ind) continue;
			} else {		
				if ("type" in accept) {
					if ("town" in goal.accept || "ind" in goal.accept || accept.type != goal.accept.type) continue;
				}
			}
        }
        return true;
    }
    return false;
}

//***************************************************************************
// Does the company have an active growth goal
// @param town_id to check.
//***************************************************************************
function CompanyData::HasGrowthGoal(town_id)
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
		if (goal.goal_type == "growth") {
			if (goal.accept.town == town_id) return true;
		}
	}
    return false;
}

//***************************************************************************
// Does the company have an active pickup goal for the given industry
//***************************************************************************
function CompanyData::HasPickupGoal(type)
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
		if (goal.goal_type == "pickup") {
			if (goal.accept.type == type) return true;
		}
	}
    return null;
}

//***************************************************************************
// Count the number of goals that ask for the given cargo type.
// @param cargo_id Cargo to check for.
// @return Number of active goals with the given cargo type.
//***************************************************************************
function CompanyData::GetNumberOfGoalsForCargo(cargo_id)
{
    local count = 0;
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
        if (goal.cargo.cid == cargo_id) count += 1;
    }
    return count;
}

//***************************************************************************
// The given industry closed, remove any goal with it.
// @param ind_id Industry that closed.
//***************************************************************************
function CompanyData::IndustryClosed(ind_id)
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
        if ("ind" in goal.accept && goal.accept.ind == ind_id) {
            goal.RemoveGoal(comp_id);
            this.active_goals[num] = null;
        }
    }
}

//***************************************************************************
// There's only major pickups
//***************************************************************************
function CompanyData::StartMonitoringPickupGoals()
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
		if (goal.goal_type == "pickup") {
			foreach (ind, _ in GSIndustryList()) {
				if (GSIndustry.GetIndustryType(ind) == goal.accept.type) {
//					Log.Info("Start Monitoring Industry: "+GSIndustry.GetName(ind)+" Cargo: "+GSCargo.GetCargoLabel(goal.cargo.cid), Log.LVL_DEBUG);
					local amount = GSCargoMonitor.GetIndustryPickupAmount(this.comp_id, goal.cargo.cid, ind, true);
				}
			}
		}
    }
}

//***************************************************************************
// Determine progress for this company's goals.
//***************************************************************************
function CompanyData::UpdateGoalsProgress()
{
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
		goal.UpdateGoalProgress(comp_id);
    }
}

//***************************************************************************
// Test whether goals of the company are 'done', and if so, drop them.
//***************************************************************************
function CompanyData::CheckAndFinishGoals()
{
	local goals_completed = 0;
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
        if (goal.CheckFinished()) {
            goal.RemoveGoal(comp_id);
			goals_completed += 1;
            this.active_goals[num] = null;
        }
    }
	return goals_completed;
}

//***************************************************************************
// Find the number of active goals that are missing for this company.
// @return Number of additional goals that the company needs.
//***************************************************************************
function CompanyData::ExpireGoals(year)
{
    local expired = 0;
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
		if (goal.goal_scope == "minor" && year >= goal.year_expiry) {

			// publish news of expiry
			local goal_news_text, destination, destination_string_news;
			switch (goal.goal_type) {
				case("growth"): 
					destination = goal.accept.town;
					destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, destination);
					goal_news_text = GSText(GSText.STR_COMPANY_GROWTH_GOAL, goal.year_expiry, goal.wanted_amount, destination_string_news);
					break;

				case("pickup"): 
					destination_string_news = this.GetIndustryTypeString(goal.accept.type);
					if (this.goal_scope == "major") {
						goal_news_text = GSText(GSText.STR_COMPANY_MAJOR_PICKUP_GOAL, goal.cargo.cid, goal.wanted_amount, destination_string_news);
					} else {
						goal_news_text = GSText(GSText.STR_COMPANY_PICKUP_GOAL, goal.year_expiry, goal.cargo.cid, goal.wanted_amount, destination_string_news);
					}
					break;
	
				case("delivery"):
					if ("town" in goal.accept) {
						destination = goal.accept.town;
						destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, destination);
					} else {
						destination = goal.accept.ind;
						destination_string_news = GSText(GSText.STR_INDUSTRY_NAME_NEWS, destination);
					}
					goal_news_text = GSText(GSText.STR_COMPANY_GOAL, goal.year_expiry, goal.cargo.cid, goal.wanted_amount, destination_string_news);
 				break;
				
				default: break;
			}
			local expiry_news_text = GSText(GSText.STR_GOAL_EXPIRY_NEWS, goal_news_text)
			goal.PublishNews(expiry_news_text, this.comp_id);

			// Once published, now remove it
			goal.RemoveGoal(comp_id);
            this.active_goals[num] = null;
			expired += 1;
		}
    }
    return expired;
}

//***************************************************************************
// Company is about to be deleted, last chance to clean up.
//***************************************************************************
function CompanyData::FinalizeCompany()
{
    foreach (num, goal in this.active_goals) {
        if (goal != null) {
            goal.RemoveGoal(comp_id);
            this.active_goals[num] = null;
        }
    }
}


//***************************************************************************
// Save company data.
//***************************************************************************
function CompanyData::SaveCompany()
{
    local result = {};
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
        result[num] <- goal.SaveGoal();
//		Log.Info("Save goal :"+num, Log.LVL_DEBUG);
    }
    return result;
}

//***************************************************************************
// Load company data from the file, constructing a new company.
// @param comp_id Company id.
// @param loaded_data Data to load for this company.
// @param cargoes Cargoes of the game.
// @return The created company.
//***************************************************************************
function CompanyData::LoadCompany(comp_id, loaded_data, cargoes)
{
    local cdata = CompanyData(comp_id);
    foreach(num, loaded_goal_data in loaded_data) {
        cdata.active_goals[num] = CompanyGoal.LoadGoal(loaded_goal_data, cargoes);
//		Log.Info("Load goal :"+num, Log.LVL_DEBUG);
    }
    return cdata;
}

//***************************************************************************
// Game data was just loaded, check whether the goals make sense.
//***************************************************************************
function CompanyData::GoalsPostLoadCheck()
{
    // Check whether the industries still live.
    foreach (num, goal in this.active_goals) {
        if (goal == null) continue;
        if ("ind" in goal.accept && !GSIndustry.IsValidIndustry(goal.accept.ind)) {
            goal.RemoveGoal(comp_id);
            this.active_goals[num] = null;
        }
    }
}


