/*
 * This file is part of Goal Based City Growth (GBCG), which is a GameScript for OpenTTD
 * Copyright (C) 2016 Patrick Brennan
 *
 * GBCG is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * GBCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GBCG; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA 02110-1301 USA.
 */

// A goal for a company.
class CompanyGoal {
    cargo = null;         // Cargo data (#Cargo)
    accept = null;        // Accepting entity.
    wanted_amount = null; // Amount to deliver for achieving the goal.
    delivered_amount = 0; // Amount delivered so far.
    goal_id = null;       // Number of the goal in OpenTTD goal window.
	start_year = null;//bob
	terraform_completion = null;//bob
    year_expiry = null;   // Year the goal will expire.
	goal_type = null;	  // "growth" or "delivery"
	goal_scope = null;	  // "major" (no expiry date, must be completed to win the game) or "minor" (has expiry date)
	
    displayed_string = null;
    displayed_count = null;

    // Construct a company goal.
    // @param comp_id Company owning the goal. Use \c null if no OpenTTD goal should be created.
    // @param cargo Cargo that should be delivered.
    // @param accept Accepting resource.
    // @param wanted_amount Amount of cargo that should be delivered to fulfil the goal.
    constructor(comp_id, cargo, accept, wanted_amount, year_expiry, goal_type, goal_scope) {
        this.cargo = cargo;
        this.accept = accept;
        this.wanted_amount = wanted_amount;
		this.year_expiry = year_expiry;
		this.goal_type = goal_type;
		this.goal_scope = goal_scope;
		
		if (this.goal_scope == "minor") this.InitYearExpiry();
		
        // Construct goal if a company id was provided.
		if (comp_id != null) {
			local goal_text, goal_news_text, gs_goal_type, destination, destination_string, destination_string_news;
			switch (this.goal_type) {
				case("growth"): 
					destination = accept.town;
					destination_string = GSText(GSText.STR_TOWN_NAME, destination);
					destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, destination);
					gs_goal_type = GSGoal.GT_TOWN;
					if (this.goal_scope == "major") {
						goal_text = GSText(GSText.STR_COMPANY_MAJOR_GROWTH_GOAL, this.wanted_amount, destination_string);
						goal_news_text = GSText(GSText.STR_COMPANY_MAJOR_GROWTH_GOAL, this.wanted_amount, destination_string_news);
					} else {
						goal_text = GSText(GSText.STR_COMPANY_GROWTH_GOAL, this.year_expiry, this.wanted_amount, destination_string);
						goal_news_text = GSText(GSText.STR_COMPANY_GROWTH_GOAL_NEWS, this.year_expiry, this.wanted_amount, destination_string_news);
					}
					break;

				case("pickup"): 
					destination = comp_id;
					destination_string = this.GetIndustryTypeString(accept.type);
					gs_goal_type = GSGoal.GT_COMPANY;
					if (this.goal_scope == "major") {
						goal_text = GSText(GSText.STR_COMPANY_MAJOR_PICKUP_GOAL, cargo.cid, this.wanted_amount, destination_string);
						goal_news_text = GSText(GSText.STR_COMPANY_MAJOR_PICKUP_GOAL, cargo.cid, this.wanted_amount, destination_string);
					} else {
						goal_text = GSText(GSText.STR_COMPANY_PICKUP_GOAL, this.year_expiry, cargo.cid, this.wanted_amount, destination_string);
						goal_news_text = GSText(GSText.STR_COMPANY_PICKUP_GOAL_NEWS, this.year_expiry, cargo.cid, this.wanted_amount, destination_string);
					}
					break;
	
				case("delivery"):
					if ("town" in this.accept) {
						destination = accept.town;
						destination_string = GSText(GSText.STR_TOWN_NAME, destination);
						destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, destination);
						gs_goal_type = GSGoal.GT_TOWN;
						// start monitoring
						local amount = GSCargoMonitor.GetTownDeliveryAmount(comp_id, cargo.cid, accept.town, true);
					} else {
						destination = accept.ind;
						destination_string = GSText(GSText.STR_INDUSTRY_NAME, destination);
						destination_string_news = GSText(GSText.STR_INDUSTRY_NAME_NEWS, destination);
						gs_goal_type = GSGoal.GT_INDUSTRY;
						// start monitoring
						local amount = GSCargoMonitor.GetIndustryDeliveryAmount(comp_id, cargo.cid, accept.ind, true);
					}
					goal_text = GSText(GSText.STR_COMPANY_GOAL, this.year_expiry, cargo.cid, this.wanted_amount, destination_string);
					goal_news_text = GSText(GSText.STR_COMPANY_GOAL_NEWS, this.year_expiry, cargo.cid, this.wanted_amount, destination_string_news);
 				break;
				
				default: break;
			}
			this.goal_id = GSGoal.New(comp_id, goal_text, gs_goal_type, destination);
			this.PublishNews(goal_news_text, comp_id);
       }
    }

    function InitYearExpiry();
    function UpdateGoalProgress(comp_id);
    function CheckFinished();
    function RemoveGoal(comp_id);
    function PublishNews(str, comp_id);
	function GetIndustryTypeString();

    function SaveGoal();
    static function LoadGoal(num, loaded_data);
};

//***************************************************************************
// Set the goal's year of expiry. Called for minor goals only.
//***************************************************************************
function CompanyGoal::InitYearExpiry()
{
    local goals_difficulty = GSController.GetSetting("goals_difficulty");
	local years = 10 - goals_difficulty + GSBase.RandRange(4);
	if (this.goal_type == "growth") years += 3;
	local date = GSDate.GetCurrentDate();
	this.year_expiry = GSDate.GetYear(date) + years;
}

//***************************************************************************
// Update the delivered amounts.
//***************************************************************************
function CompanyGoal::UpdateGoalProgress(comp_id)
{
    local perc, remaining, progress_text, delivered, destination_string_news;
	switch (this.goal_type) {
		case("growth"): 
			this.delivered_amount = GSTown.GetPopulation(this.accept.town);
			if (this.goal_id != null) {
				if (this.delivered_amount >= this.wanted_amount) {
					remaining = 0;
					GSGoal.SetCompleted(this.goal_id, true);
					destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, this.accept.town);
					local goal_won_news = GSText(GSText.STR_COMPANY_GROWTH_GOAL_WON_NEWS, this.wanted_amount, destination_string_news);
					this.PublishNews(goal_won_news, comp_id);
				} else {
					remaining = this.wanted_amount - this.delivered_amount;
					if (remaining < 0) remaining = 2;
				}
				progress_text = GSText(GSText.STR_GROWTH_PROGRESS, remaining);
				GSGoal.SetProgress(this.goal_id, progress_text);
			}
		break;

		case("pickup"): 
			// first deduct a twelth to simulate losing the delivery from 12 months ago
			local averaged_month_float = this.delivered_amount.tofloat() / 12;
			this.delivered_amount -= averaged_month_float.tointeger();
			delivered = 0;
			foreach (ind, _ in GSIndustryList()) {
				if (GSIndustry.GetIndustryType(ind) == this.accept.type) {
					delivered += GSCargoMonitor.GetIndustryPickupAmount(comp_id, this.cargo.cid, ind, true);
				}
			}
			local averaged_month_float = delivered.tofloat() / 12;
			this.delivered_amount += averaged_month_float.tointeger();
//			Log.Info("Monitoring: "+GSIndustryType.GetName(this.accept.type)+" Cargo: "+GSCargo.GetCargoLabel(this.cargo.cid)+"Delivered: "+delivered+" New Total: "+this.delivered_amount, Log.LVL_DEBUG);
			if (this.delivered_amount > 0) {
				if (this.goal_id != null) {
					if (this.delivered_amount >= this.wanted_amount) {
						perc = 100;
						GSGoal.SetCompleted(this.goal_id, true);
						destination_string_news = this.GetIndustryTypeString(this.accept.type);
						local goal_won_news = GSText(GSText.STR_COMPANY_PICKUP_GOAL_WON_NEWS, cargo.cid, this.wanted_amount, destination_string_news);
						this.PublishNews(goal_won_news, comp_id);
					} else {
						perc = 100 * this.delivered_amount / this.wanted_amount;
						if (perc > 100) perc = 100;
					}
					progress_text = GSText(GSText.STR_PROGRESS, perc);
					GSGoal.SetProgress(this.goal_id, progress_text);
				}
			}
		break;
		
		case("delivery"):
			if ("town" in this.accept) {
				delivered = GSCargoMonitor.GetTownDeliveryAmount(comp_id, this.cargo.cid, this.accept.town, true);
							destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, this.accept.town);
			} else {
				delivered = GSCargoMonitor.GetIndustryDeliveryAmount(comp_id, this.cargo.cid, this.accept.ind, true);
							destination_string_news = GSText(GSText.STR_INDUSTRY_NAME_NEWS, this.accept.ind);
			}
			if (delivered > 0) {
				this.delivered_amount += delivered;
				if (this.goal_id != null) {
					if (this.delivered_amount >= this.wanted_amount) {
						perc = 100;
						GSGoal.SetCompleted(this.goal_id, true);
						if ("town" in this.accept) {
							destination_string_news = GSText(GSText.STR_TOWN_NAME_NEWS, this.accept.town);
						} else {
							destination_string_news = GSText(GSText.STR_INDUSTRY_NAME_NEWS, this.accept.ind);
						}
						local goal_won_news = GSText(GSText.STR_COMPANY_GOAL_WON_NEWS, cargo.cid, this.wanted_amount, destination_string_news);
						this.PublishNews(goal_won_news, comp_id);
					} else {
						perc = 100 * this.delivered_amount / this.wanted_amount;
						if (perc > 100) perc = 100;
					}
					progress_text = GSText(GSText.STR_PROGRESS, perc);
					GSGoal.SetProgress(this.goal_id, progress_text);
				}
			}
		break;

		default: break;
	}
}

//***************************************************************************
// Test whether the goal can be considered 'done' (or obsolete).
// @return Whether the goal is considered done.
//***************************************************************************
function CompanyGoal::CheckFinished()
{
    return this.delivered_amount >= this.wanted_amount;
}

//***************************************************************************
// Goal is considered 'done', last chance to clean up before the goal is dropped
// (to make room for a new goal).
//***************************************************************************
function CompanyGoal::RemoveGoal(comp_id)
{
    if (this.goal_id != null) {
		switch (this.goal_type) {
			case("delivery"): 
				if ("town" in this.accept) {
					local delivered = GSCargoMonitor.GetTownDeliveryAmount(comp_id, this.cargo.cid, this.accept.town, false);
				} else {
					local delivered = GSCargoMonitor.GetIndustryDeliveryAmount(comp_id, this.cargo.cid, this.accept.ind, false);
				}
				break;
			case("pickup"): 
				foreach (ind, _ in GSIndustryList()) {
					if (GSIndustry.GetIndustryType(ind) == this.accept.type) {
						local amount = GSCargoMonitor.GetIndustryPickupAmount(comp_id, this.cargo.cid, ind, false);
					}
				}
				break;
			default: break;
		}
		GSGoal.Remove(this.goal_id);
	}
}

//***************************************************************************
// Publish a news item about the goal.
// @param news_text String with text to publish.
// @param comp_id Company owning the goal.
//***************************************************************************
function CompanyGoal::PublishNews(news_text, comp_id)
{
	GSNews.Create(GSNews.NT_GENERAL, news_text, comp_id);
}

//***************************************************************************
// Load an existing goal.
//***************************************************************************
function CompanyGoal::SaveGoal()
{
    return {cid=this.cargo.cid, accept=this.accept, wanted=this.wanted_amount,
            delivered=this.delivered_amount, goal=this.goal_id, year_expiry=this.year_expiry, goal_type=this.goal_type, goal_scope=this.goal_scope};
}

//***************************************************************************
// Load an existing goal.
// @param loaded_data Data of the goal.
// @param cargoes Cargoes of the game.
// @return The loaded goal, if loading went ok.
//***************************************************************************
function CompanyGoal::LoadGoal(loaded_data, cargoes)
{
    local goal = null;
	foreach (cargo_num, cargo in cargoes) {
		if (cargo.cid == loaded_data.cid) {
			goal = CompanyGoal(null, cargo, loaded_data.accept, loaded_data.wanted, loaded_data.year_expiry, loaded_data.goal_type, loaded_data.goal_scope);
			goal.delivered_amount = loaded_data.delivered;
			goal.goal_id = loaded_data.goal;
			goal.year_expiry = loaded_data.year_expiry;
			goal.goal_type = loaded_data.goal_type;
			goal.goal_scope = loaded_data.goal_scope;
			return goal;
		}
	}
    return null;
}

//***************************************************************************
// Set the industry type name, which is done by brute force because there's no friggin' {INDUSTRY_TYPE} special string
// and special string IDs can't have spaces in them, so you can't create string IDs in english.txt based on industry name
//***************************************************************************
function CompanyGoal::GetIndustryTypeString(type)
{
	local name = GSIndustryType.GetName(type);
	local typetext = null;
	switch (name) {
		case("Arable Farm"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_ARABLEFARM); 
			break;
		case("Aluminium Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_ALUMINIUMPLANT); 
			break;
		case("Bank"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_BANK); 
			break;
		case("Biorefinery"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_BIOREFINERY); 
			break;
		case("Brewery"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_BREWERY); 
			break;
		case("Brick Works"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_BRICKWORKS); 
			break;
		case("Builders Yard"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_BUILDERSYARD); 
			break;
		case("Bulk Terminal"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_BULKTERMINAL); 
			break;
		case("Cement Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_CEMENTPLANT); 
			break;
		case("Chemical Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_CHEMICALPLANT); 
			break;		
		case("Cider Mill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_CIDERMILL); 
			break;
		case("Clay Pit"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_CLAYPIT); 
			break;
		case("Coal Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_COALMINE); 
			break;
		case("Coffee Estate"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_COFFEEESTATE); 
			break;
		case("Copper Ore Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_COPPEROREMINE); 
			break;
		case("Copper Refinery"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_COPPERREFINERY); 
			break;
		case("Dairy"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_DAIRY); 
			break;
		case("Dairy Farm"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_DAIRYFARM); 
			break;
		case("Diamond Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_DIAMONDMINE); 
			break;
		case("Dredging Site"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_DREDGINGSITE); 
			break;
		case("Factory"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FACTORY); 
			break;
		case("Farm"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FARM); 
			break;
		case("Fertiliser Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FERTILISERPLANT); 
			break;
		case("Fishing Grounds"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FISHINGGROUNDS); 
			break;
		case("Fishing Harbour"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FISHINGHARBOUR); 
			break;
		case("Flour Mill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FLOURMILL); 
			break;
		case("Food Processing Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FOODPROCESSINGPLANT); 
			break;
		case("Forest"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FOREST); 
			break;
		case("Fruit Plantation"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FRUITPLANTATION); 
			break;
		case("Furniture Factory"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FURNITUREFACTORY); 
			break;
		case("General Store"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_GENERALSTORE); 
			break;
		case("Glass Works"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_GLASSWORKS); 
			break;
		case("Gold Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_GOLDMINE); 
			break;
		case("Grocer's Shop"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_GROCERSSHOP); 
			break;
		case("Hardware Store"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_HARDWARESTORE); 
			break;
		case("Hotel"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_HOTEL); 
			break;
		case("Iron Ore Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_IRONOREMINE); 
			break;
		case("Iron Works"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_IRONWORKS); 
			break;
		case("Lime Kiln"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_LIMEKILN); 
			break;
		case("Liquids Terminal"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_LIQUIDSTERMINAL); 
			break;
		case("Lumber Mill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_LUMBERMILL); 
			break;
		case("Machine Shop"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_MACHINESHOP); 
			break;
		case("Manganese Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_MANGANESEMINE); 
			break;
		case("Metal Fabrication Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_METALFABRICATIONPLANT); 
			break;
		case("Metal Workshop"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_METALWORKSHOP); 
			break;
		case("Mixed Farm"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_MIXEDFARM); 
			break;
		case("Nitrate Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_NITRATEMINE); 
			break;
		case("Oil Refinery"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_OILREFINERY); 
			break;
		case("Oil Rig"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_OILRIG); 
			break;
		case("Oil Wells"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_OILWELLS); 
			break;
		case("Orchard and Piggery"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_ORCHARDANDPIGGERY); 
			break;
		case("Paper Mill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PAPERMILL); 
			break;
		case("Phosphate Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PHOSPHATEMINE); 
			break;		
		case("Plastics Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PLASTICSPLANT); 
			break;
		case("Port"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PORT); 
			break;		
		case("Power Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_POWERPLANT); 
			break;		
		case("Power Station"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_POWERSTATION); 
			break;
		case("Printing Works"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PRINTINGWORKS); 
			break;
		case("Pyrite Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PYRITEMINE); 
			break;		
		case("Pyrite Smelter"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_PYRITESMELTER); 
			break;		
		case("Quarry"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_QUARRY); 
			break;
		case("Ranch"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_RANCH); 
			break;
		case("Recycling Depot"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_RECYCLINGDEPOT); 
			break;
		case("Recycling Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_RECYCLINGPLANT); 
			break;
		case("Rubber Plantation"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_RUBBERPLANTATION); 
			break;
		case("Sawmill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_SAWMILL); 
			break;
		case("Service Station"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_SERVICESTATION); 
			break;
		case("Sheep Farm"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_SHEEPFARM); 
			break;
		case("Smithy Forge"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_SMITHYFORGE); 
			break;
		case("Steel Mill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_STEELMILL); 
			break;
		case("Stockyard"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_STOCKYARD); 
			break;
		case("Sugar Refinery"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_SUGARREFINERY); 
			break;
		case("Supply Yard"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_SUPPLYYARD); 
			break;
		case("Textile Mill"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_TEXTILEMILL); 
			break;
		case("Timber Yard"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_TIMBERYARD); 
			break;
		case("Trading Post"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_TRADINGPOST); 
			break;		
		case("Tyre Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_TYREPLANT); 
			break;
		case("Vehicle Factory"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_VEHICLEFACTORY); 
			break;
		case("Vineyard"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_VINEYARD); 
			break;
		case("Water Supply"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_WATERSUPPLY); 
			break;		
		case("Water Tower"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_WATERTOWER); 
			break;
		case("Wrecking Yard"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_WRECKINGYARD); 
			break;
		//mars
		case("Air Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_AIRMINE); 
			break;
		case("Aquifer Pump"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_AQUIFERPUMP); 
			break;
		case("Electrolysis Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_ELECTROLYSISPLANT); 
			break;
		case("Chemical Factory"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_CHEMICALFACTORY); 
			break;
		case("Goods Factory"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_GOODSFACTORY); 
			break;
		case("Fusion Power Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_FUSIONPOWERPLANT); 
			break;		
		case("Ice Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_ICEMINE); 
			break;
		case("Metal Works"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_METALWORKS); 
			break;
		case("Ore Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_OREMINE); 
			break;
		case("Regolith Mine"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_REGOLITHMINE); 
			break;
		case("Regolith Processing"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_REGOLITHPROCESSING); 
			break;
		case("Terraforming Plant"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_TERRAFORMINGPLANT); 
			break;
		case("Life Support Centre"): 
			typetext = GSText(GSText.STR_INDUSTRYTYPE_LIFESUPPORTCENTRE); 
			break;
		default:
			Log.Info("CompanyGoal.GetIndustryTypeString - Missing string for: "+name, Log.LVL_DEBUG);
			typetext = GSText(GSText.STR_INDUSTRYTYPE_NOTFOUND);
			break;
	}
    return GSText(GSText.STR_INDUSTRYTYPE, typetext);
}
