/*
 * This file is part of Goal Based City Growth (GBCG), which is a GameScript for OpenTTD
 * Copyright (C) 2016 Patrick Brennan
 *
 * GBCG is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * GBCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GBCG; If not, see <http://www.gnu.org/licenses/> or
 * write to the Free Software Foundation, Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA 02110-1301 USA.
 */

require("version.nut");
require("cargo.nut");
require("company.nut");
require("companygoal.nut");
require("town.nut");
require("story.nut");
require("strings.nut");

// Import SuperLib for GameScript
import("util.superlib", "SuperLib", 38);
Log <- SuperLib.Log;
Helper <- SuperLib.Helper;


class MainClass extends GSController
{

    cargoes = null;  ///< Cargoes of the game (index -> 'cid' number, 'freight' boolean, 'effect' on town).
    num_cargoes = 0; ///< Number of cargoes in 'this.cargoes'.
    sum_weight = 0;  ///< Total sum of the weights of the cargoes.
	companies = null;
	towns = null;
	current_date = null;
	current_month = null;
	current_year = null;
	start_year = null;//bob
	terraform_completion = null;//bob
	gs_init_done = null;
	load_saved_data = null;
	current_save_version = null;
	goals_completed_total = null;
	goals_expired_total = null;
	goals_total_id = null;
	goals_game_won = null;

	constructor() {
		this.towns = null;
		this.current_date = 0;
		this.current_month = 0;
		this.current_year = 0;
		this.goals_completed_total = 0;
		this.goals_expired_total = 0;
		this.terraform_completion = 0;//bob
		this.start_year = 0;//bob
		this.gs_init_done = false;
		this.load_saved_data = false;
		this.goals_game_won = false;
		this.current_save_version = 6;    // Ensures compatibility between revisions
		::TownDataTable <- {};
		::CompanyDataTable <- {};
		::SettingsTable <- {
			industry_NewGRF = GSController.GetSetting("industry_NewGRF"),
			use_town_sign = GSController.GetSetting("use_town_sign")
		        
		}
	}
}

//***************************************************************************
//***************************************************************************
function MainClass::Init()
{
	// Check game settings
	GSGameSettings.SetValue("economy.town_growth_rate", 2);
	GSGameSettings.SetValue("economy.fund_buildings", 0);

    // Set current date
	this.current_date = GSDate.GetCurrentDate();
	this.current_month = GSDate.GetMonth(this.current_date);
	this.current_year = GSDate.GetYear(this.current_date);
	
	if(!load_saved_data)
	{
		this.start_year = GSDate.GetYear(this.current_date);//bob
	}
	
	// Initialize cargo lists and variables for town growth requirements
	InitCargoLists();

	// Check whether saved data is in the current save format.
	if (!load_saved_data) Helper.ClearAllSigns();

	// Create the towns list
	Log.Info("Create town list ... (can take a while on large maps)", Log.LVL_INFO);
	this.towns = this.InitTownList();

 // Initialize if not not already initialized during load process   
	if (this.companies == null) { 
		// Examine and store cargo types of the game.
		this.cargoes = {};
		this.num_cargoes = 0;
		this.sum_weight = 0;
		for (local cid = 0; cid < 32; cid += 1) {
			if (!GSCargo.IsValidCargo(cid)) continue;
			local cargo = Cargo(this.num_cargoes, cid, GSCargo.IsFreight(cid),  GSCargo.GetTownEffect(cid));
			this.cargoes[this.num_cargoes] <- cargo;
			this.num_cargoes += 1;
			this.sum_weight += cargo.weight;
		}

		// Construct empty companies.
		this.companies = {};
		for (local comp_id = GSCompany.COMPANY_FIRST; comp_id <= GSCompany.COMPANY_LAST; comp_id++) {
			this.companies[comp_id] <- null;
		}
	}
	
	// Ending initialization
	this.gs_init_done = true;

	// Now we can free ::TownDataTable and ::CompanyDataTable for the save/load processes
	::TownDataTable = null;
	::CompanyDataTable = null;
}

//***************************************************************************
//***************************************************************************
function MainClass::Start()
{
	// Initializing the script
	local start_tick = GSController.GetTick();
	GSGame.Pause();
	Log.Info("Script initialisation...", Log.LVL_INFO);
	this.Init();
	GSGame.Unpause();

	local setup_duration = GSController.GetTick() - start_tick;
	Log.Info("Setup took " + setup_duration + " ticks.", Log.LVL_DEBUG);

	// Wait for the game to start
	GSController.Sleep(1);

	// Create and fill StoryBook. This can't be done before OTTD is ready.
	if (!load_saved_data) {
		local story_editor = StoryEditor();
		story_editor.CreateStoryBook();
	}

	// Construct global goal, which is just an ongoing progress statement on how many goals have been completed / expired
	if (!this.load_saved_data) {
		
		local industry_setting = GSController.GetSetting("industry_NewGRF");
	
		Log.Info("Industry setting: " + industry_setting, Log.LVL_DEBUG);
		if(industry_setting == 9)//bob mars
		{
			local temp_goal_text = GSText(GSText.STR_GOAL_TOTALS_MARS,this.terraform_completion / 50, this.goals_completed_total, this.goals_expired_total);
			this.goals_total_id = GSGoal.New(GSCompany.COMPANY_INVALID, temp_goal_text, GSGoal.GT_NONE, 0);
		}
		else
		{
			local temp_goal_text = GSText(GSText.STR_GOAL_TOTALS, this.goals_completed_total, this.goals_expired_total);
			this.goals_total_id = GSGoal.New(GSCompany.COMPANY_INVALID, temp_goal_text, GSGoal.GT_NONE, 0);
		}	
	}

   	if (this.load_saved_data) { // Script data was loaded.
       	this.CompaniesPostLoadCheck();
   	}

// Required to create company first so goals won't crash
    local result = this.ProcessEvents();

// Establish initial major town growth goals
	if (!load_saved_data) {
		local new_goals = GSController.GetSetting("start_major_town_goals");
		if (new_goals > 0) {
			foreach (comp_id, cdata in this.companies) {
				if (cdata == null) continue;
				for (local num = 0; num < new_goals; num += 1) {
					local result = this.CreateMajorTownChallenge(comp_id);
				}
			}
		}
	}

// Establish initial major cargo pickup/delivery goals
	if (!load_saved_data) {
		local new_goals = GSController.GetSetting("start_major_cargo_goals");
		if (new_goals > 0) {
			foreach (comp_id, cdata in this.companies) {
				if (cdata == null) continue;
				for (local num = 0; num < new_goals; num += 1) {
					local result = this.CreateMajorCargoChallenge(comp_id);
				}
			}
		}
	}
	foreach (comp_id, cdata in this.companies) {
	    if (cdata == null) continue;
		cdata.StartMonitoringPickupGoals();
	}
	
// Establish initial random goals
	if (!load_saved_data) {
		local max_new_goals = GSController.GetSetting("minor_goals_per_year");
		if (max_new_goals != 0) {
			local new_goals = max_new_goals;//GSBase.RandRange(max_new_goals);//bob always have initial goals
			if (new_goals > 0) {
				foreach (comp_id, cdata in this.companies) {
					if (cdata == null) continue;
					for (local num = 0; num < new_goals; num += 1) {
						this.CreateChallenge(comp_id);
					}
				}
			}
		}
	}

// Main loop - perform once per day
	while (true) {
      
		local result = this.ProcessEvents();

		this.ManageTownsAndGoals();

		this.Sleep(74);
	}
}


//***************************************************************************
// For each town, an instance of GoalTown is created to store the data related to that town.
//***************************************************************************
function MainClass::InitTownList()
{
	local towns_list = GSTownList();
	local towns_array = [];

	foreach (t, _ in towns_list) {
		towns_array.append(GoalTown(t, this.load_saved_data));
	}

	return towns_array;
}

//***************************************************************************
// Function called on town creation. We need to add a now GoalTown
// instance to the this.towns array. The array order doesn't matter,
// since we never use the array index, only its values.
//***************************************************************************

function MainClass::UpdateTownList(town_id)
{
	this.towns.append(GoalTown(town_id, false));
}

///***************************************************************************
// Function called each day to manage towns and goals.
//***************************************************************************
function MainClass::ManageTownsAndGoals()
{
// Run the daily functions
	local date = GSDate.GetCurrentDate();
	local diff_date = date - this.current_date;
	if (diff_date == 0) return;
	else {
		this.current_date = date;
	}

	local industry_setting = GSController.GetSetting("industry_NewGRF");
					
// Run the monthly functions
	local month = GSDate.GetMonth(date);
	local diff_month = month - this.current_month;
	if (diff_month == 0) return;
	else {
		local month_tick = GSController.GetTick();
		Log.Info("Starting Monthly Updates...", Log.LVL_INFO);
		
		local tree_frame = 0;
		
		// Update goals 
        foreach (comp_id, cdata in this.companies) {
	       	if (cdata == null) continue;
			cdata.UpdateGoalsProgress();
           	local goals_completed = cdata.CheckAndFinishGoals();
			this.goals_completed_total += goals_completed;
			
			
			// Each month, lose 1% of your balance for every expired goal
			if (this.goals_expired_total >= 0) {//bob always triggers
			   	local balance = GSCompany.GetBankBalance(comp_id);
				if (balance > 0) {
					local penalty_percentage = 0.01 * this.goals_expired_total;
					if (penalty_percentage > 1) penalty_percentage = 1;
					local penaltyfloat = balance * penalty_percentage * -1;
					local penalty = penaltyfloat.tointeger();
					
					if(penalty > 0) {
						Log.Info("Penalty Balance: "+balance+"  Penalty: "+penalty, Log.LVL_INFO);
						local result=GSCompany.ChangeBankBalance(comp_id,penalty,GSCompany.EXPENSES_OTHER);
					}

					//bob tree planting time

					local tree_cost = GSTile.GetBuildCost(GSTile.BT_BUILD_TREES);
					local tree_count = 0;
					local terraforming = 0;
					local chem_total = 0;
					local orgn_total = 0;

					if(industry_setting == 9)//bob mars
					{
						//find terraforming plants
						foreach (ind, _ in GSIndustryList_CargoAccepting(5)) {
							 
							foreach (ind2, _ in GSIndustryList_CargoAccepting(7)) {
								
								if(ind == ind2)
								{
									// GSLog.Info("Industry " + GSIndustry.GetName(ind) + " found");
									local loc = GSIndustry.GetLocation(ind);
									
									local chem = GSCargoMonitor.GetIndustryDeliveryAmount(comp_id, 5, ind, true);
									local orgn = GSCargoMonitor.GetIndustryDeliveryAmount(comp_id, 7, ind, true);
									
									chem_total += chem;
									orgn_total += orgn;
									
								}
								
								// dests[num_dests] <- {ind=ind, loc=loc};
								// num_dests += 1;
							}
						}
						
						// GSLog.Info("Industry chem: " + chem_total );
						// GSLog.Info("Industry orgn: " + orgn_total );
						
						if(chem_total > 4 && orgn_total > 4)
						{

							local terra_total = chem_total*orgn_total;
							local tree_total = 1;
							
							terra_total = 1 + terra_total / 3000;
							
							if(terra_total < 1)
							{
								terra_total = 1;
							}
							
							if(!this.goals_game_won)
							{
								this.terraform_completion = terraform_completion + terra_total;
								// GSLog.Info("Terraform increment: " + terra_total + " total:" + terraform_completion);
								tree_total = terra_total;
								
								if(tree_frame < 21)
								{
									for (local attempt = 1; attempt <= tree_total; attempt += 1) {
										
										local company_result = GSCompanyMode(comp_id);//fixme
									
										local empty_tile = _SuperLib_Tile.GetRandomTile();

										if(tree_frame < 21)
										{
											if(GSTile.IsBuildable(empty_tile))
											{
												if(!GSTile.PlantTree(empty_tile)) 
												{
													// Log.Info("Tree failed."+ attempt, Log.LVL_INFO);
												}
												else
												{
													// Log.Info("Tree planted:" + attempt + " cost:" + tree_cost, Log.LVL_INFO);
													tree_count = tree_count + 1
													tree_frame = tree_frame + 1

												}
											}
										}
									}
									
									if(tree_count > 0)
									{
										if(!GSCompany.ChangeBankBalance(comp_id,tree_cost*tree_count,GSCompany.EXPENSES_OTHER))
										{
											// Log.Info("Transaction error?", Log.LVL_INFO);	
										}
										else
										{
											Log.Info("Terraform bonus credits:" + tree_cost*tree_count*2, Log.LVL_DEBUG)//get double back
										}
									}
								}
							}
						}
						
						
					}
					
				}
			}			
		
			// Determine if game won - no major goals outstanding, completed goals > target, and > expired goals
			if (!this.goals_game_won) {
				local num_goals_major = cdata.GetNumberOfGoals("major");
				if (num_goals_major == 0) {
					local new_goals = GSController.GetSetting("minor_goals_per_year");
					local target;
					if (new_goals == 0) {
						target = 0;
					} else {
						local difficulty = GSController.GetSetting("goals_difficulty");
						target = difficulty * 10;
					}
					
					if(industry_setting == 9)//bob mars
					
						if (num_goals_major == 0 && this.terraform_completion >= 5000)
							this.goals_game_won = true;
						
					else
					{
						
						if (num_goals_major == 0 && this.goals_completed_total >= target && this.goals_completed_total > this.goals_expired_total)
							this.goals_game_won = true;
						
					}
				}
			}
			
			
			local temp_goal_text;	
			if (this.goals_game_won) {
				temp_goal_text = GSText(GSText.STR_GOAL_TOTALS_CONGRATULATIONS, this.goals_completed_total, this.goals_expired_total);
			} else {
				
				if(industry_setting == 9)//bob mars
				{
					temp_goal_text = GSText(GSText.STR_GOAL_TOTALS_MARS,this.terraform_completion / 50, this.goals_completed_total, this.goals_expired_total);
				}
				else
				{
					temp_goal_text = GSText(GSText.STR_GOAL_TOTALS, this.goals_completed_total, this.goals_expired_total);
				}
			}
			GSGoal.SetText(goals_total_id, temp_goal_text);
		}

		foreach (town in this.towns) {
			town.MonthlyManageTown();
		}

		this.current_month = month;
		local month_tick_duration = GSController.GetTick() - month_tick;
		
		Log.Info("Monthly Update took "+month_tick_duration+" ticks.", Log.LVL_DEBUG);
		
	}

// Run the yearly functions
	local year = GSDate.GetYear(date);
	local diff_year = year - this.current_year;
	if (diff_year == 0) return;
	else {
		local year_tick = GSController.GetTick();
		Log.Info("Starting Yearly Updates...", Log.LVL_DEBUG);
		
		foreach (comp_id, cdata in this.companies) {
	        if (cdata == null) continue;

			// Expire goals with a year <= this year
			local num_goals_expired = cdata.ExpireGoals(year);
			this.goals_expired_total += num_goals_expired;
			local temp_goal_text;
			if (!this.goals_game_won) {
				
				if(industry_setting == 9)//mars
				{
					temp_goal_text = GSText(GSText.STR_GOAL_TOTALS_MARS,this.terraform_completion / 50, this.goals_completed_total, this.goals_expired_total);
				}
				else
				{
					temp_goal_text = GSText(GSText.STR_GOAL_TOTALS, this.goals_completed_total, this.goals_expired_total);
				}
				
			} else {
				temp_goal_text = GSText(GSText.STR_GOAL_TOTALS_CONGRATULATIONS, this.goals_completed_total, this.goals_expired_total);
			}
			GSGoal.SetText(goals_total_id, temp_goal_text);

			// Generate new goals for the year based on the parameter setting
			// But allow the player to manipulate whether he wants more goals or not
			local stop_minor_goals = false;
			local num_create_major_town_goals = 0;
			local num_create_major_cargo_goals = 0;
			if (cdata != null) {
				// do this inside an irrelevant if to ensure the companymode instance is destroyed on exit
				// because otherwise it GSGoal stops working properly
				local company_result = GSCompanyMode(comp_id);
				foreach (sign, _ in GSSignList()) {
					if (GSSign.GetName(sign) == "stop minor goals") stop_minor_goals = true;
					if (GSSign.GetName(sign) == "create major town goal") num_create_major_town_goals += 1;
					if (GSSign.GetName(sign) == "create major cargo goal") num_create_major_cargo_goals += 1;
				}
			}
			
			if (num_create_major_town_goals > 0) {
				for (local attempt = 1; attempt <= num_create_major_town_goals; attempt += 1) {
					local goal_result = this.CreateMajorTownChallenge(comp_id);
				}
			}
			if (num_create_major_cargo_goals > 0) {
				for (local attempt = 1; attempt <= num_create_major_cargo_goals; attempt += 1) {
					local goal_result = this.CreateMajorCargoChallenge(comp_id);
				}
			}
						
			if (!stop_minor_goals) {
				
				if(GSController.GetSetting("minor_goals_per_year") > 0)
				{
					//bob goal chance
			
					local max_new_goals = GSController.GetSetting("minor_goals_per_year") + 1;
				
					local goal_year = GSBase.RandRange(20);//bob reduced goals count
					
					local goal_chances = GSController.GetSetting("minor_goal_chance");
					local max_years = 0;
					max_years = this.current_year - this.start_year;
					
					local max_expireds = 0;
					max_expireds = this.goals_expired_total;
					
					if(max_expireds < 0)
					{
						max_expireds = 0;
					}
					if(max_expireds > 8)
					{
						max_expireds = 8;
					}
					
					if(max_years < 1)
					{
						max_years = 1;
					}
					
					if(max_years > 100)
					{
						max_years = 100;
					}
					
					if(max_expireds > (max_years / 8) + 1)//expiry cannot become better than initial start difficulty
					{
						// Log.Info("Expired goals limit: " + max_expireds + " to:" + ((max_years / 8) + 1), Log.LVL_INFO);
						max_expireds = (max_years / 8) + 1;
					}
					
					goal_year = goal_year + ((max_years / 8) - (max_expireds));//bob slight increase in minor goals over 100 years
					
					// Log.Info("Yearly goal chance:"+goal_year+", year_add:"+ (max_years / 8) + ", expired_bonus:" + max_expireds + ", years elapsed:" + max_years, Log.LVL_INFO);
					
					if(max_years != 1) {
						if(goal_year > goal_chances) {
							local new_goals = GSBase.RandRange(max_new_goals);//bob
							
							if(new_goals < 1)
								new_goals = 1;
							
							for (local num = 0; num < new_goals; num += 1) {
								this.CreateChallenge(comp_id);
							}
						}
					}
				}
			}
		}
				
// Then, finish up		
		this.current_year = year;
		local year_tick_duration = GSController.GetTick() - year_tick;
		Log.Info("Yearly update took "+year_tick_duration+" ticks.", Log.LVL_DEBUG);
	}

}

//***************************************************************************
// Adds a major town growth goal for a company
//***************************************************************************
function MainClass::CreateMajorTownChallenge(comp_id)
{
    local cdata = this.companies[comp_id];
	local cargo_index = null;
	local goal_type = "growth";
	
	cargo_index = this.SelectCargo(goal_type);
	local cargo = this.cargoes[cargo_index];
	if (cargo_index < 0) return;
			
	local attempts = 0;
	while (attempts <= 10) {
		local accept = FindGrowthChallenge(cargo_index, comp_id);
		if (accept != null) {
			local goals_difficulty_setting = GSController.GetSetting("goals_difficulty");
			local amount = (goals_difficulty_setting + 1 ) * 10000;
			local dest_name = GSTown.GetName(accept.town);
			if (cdata != null) {
				cdata.AddActiveGoal(cargo, accept, amount, null, goal_type, "major");
				return true;
			}
		}
		attempts += 1;
	}
	return false;
}

//***************************************************************************
// Adds a major cargo pickup/delivery goal for a company 
//***************************************************************************
function MainClass::CreateMajorCargoChallenge(comp_id)
{
    local cdata = this.companies[comp_id];
	local goal_type = "pickup";
    local dests = {};

	// Determine how many processing industries there are, and then pick one randomly
	local industrytypes_processing = {};
	local industrytypes_raw = {};
	local num_raw = 0;
	foreach (type, _ in GSIndustryTypeList()) {
		local name=GSIndustryType.GetName(type);
//		Log.Info("INDUSTRYTYPE: "+name, Log.LVL_DEBUG);
		if (GSIndustryType.IsRawIndustry(type)) {
			if (cdata != null && cdata.HasPickupGoal(type) == null) {
				industrytypes_raw[num_raw] <- (type);
				num_raw += 1;
//				Log.Info("Raw Industry: "+GSIndustryType.GetName(type), Log.LVL_DEBUG);
			}
		}
	}
	
	if (num_raw == 0) return;
	
	// Try 3 times to get a cargo that's producing >0 cargo
	local cargo_index, wanted_cargo, production, amountfloat, amount;
	local max_attempts = 3;
	local primary_goal_created = false;
	for (local attempt = 1; attempt <= max_attempts; attempt += 1) {
		local chosen_industrytype = industrytypes_raw[GSBase.RandRange(num_raw)];
		Log.Info("Chosen Raw Industry: "+GSIndustryType.GetName(chosen_industrytype), Log.LVL_INFO);

		// Pick a cargo that the chosen industry type generates, 
		// determine total production per month across the map for this industry type / cargo combination,
		// and create a major pickup goal for it
		cargo_index = this.SelectOutputCargo(chosen_industrytype);
		wanted_cargo = this.cargoes[cargo_index];
		production = GetProductionForIndustryTypeCargo(chosen_industrytype, wanted_cargo.cid);
		local major_scale = GSController.GetSetting("major_scale_factor");
		amountfloat = (production.tofloat() * (major_scale/100));//bob higher industry requirement
		amount = amountfloat.tointeger();
		if (amount != 0) {
			dests[0] <- {type=chosen_industrytype, loc=chosen_industrytype};
			local accept = dests[0];
			if (cdata != null) {
				cdata.AddActiveGoal(wanted_cargo, accept, amount, null, goal_type, "major");
			}
			attempt = 3;
			primary_goal_created = true;
		}
	}
	if (!primary_goal_created) return;
	
	// Create x vertically integrated goals, where each new goal finds an industry that uses the output from the previous goal 
	// as an input cargo, determine the total production for that cargo across the map, and generate a goal for it.
	local max_vertical_goals = GSController.GetSetting("start_major_cargo_vertical_goals");
	for (local attempt = 1; attempt <= max_vertical_goals; attempt += 1) {
		// Find a vertically integrated industry, that uses the output cargo from the above goal as an input cargo
		local eligible_industrytypes = {};
		local eligible = 0;
		local input_cargo = wanted_cargo.cid;
		foreach (type, _ in GSIndustryTypeList()) {
			if (!GSIndustryType.IsRawIndustry(type)) {
				if (GSIndustryType.IsProcessingIndustry(type) || attempt > 1) {
					if (cdata != null && cdata.HasPickupGoal(type) == null) {
						foreach (accepted_cargo, _ in GSIndustryType.GetAcceptedCargo(type)) {
							if (accepted_cargo == input_cargo) {
								local num_industries_for_this_type = 0;
								foreach (ind, _ in GSIndustryList()) {
									if (GSIndustry.GetIndustryType(ind) == type) num_industries_for_this_type += 1;
								}
								if (num_industries_for_this_type > 0) {
									Log.Info("Vertical Eligible Industry Type: "+GSIndustryType.GetName(type), Log.LVL_DEBUG);
									eligible_industrytypes[eligible] <- (type);
									eligible += 1;
								}
							}
						}
					}
				}
			}
		}
		if (eligible > 0) {
			// Set production amount required as 0.75 of the previous vertical goal
			local chosen_industrytype = eligible_industrytypes[GSBase.RandRange(eligible)];
			Log.Info("Chosen Vertical Industry: "+GSIndustryType.GetName(chosen_industrytype), Log.LVL_DEBUG);
			cargo_index = this.SelectOutputCargo(chosen_industrytype);
			if (cargo_index != null) {
				wanted_cargo = this.cargoes[cargo_index];
				amountfloat = amount.tofloat() * 0.75;
				amount = amountfloat.tointeger();
				dests[0] <- {type=chosen_industrytype, loc=chosen_industrytype};
				local accept = dests[0];
				if (cdata != null) cdata.AddActiveGoal(wanted_cargo, accept, amount, null, goal_type, "major");
			} else {
				attempt = max_vertical_goals;
			}
		} else {
			attempt = max_vertical_goals;
		}
	}
	
//	Log.Info("CreateMajorCargoChallenge finished", Log.LVL_DEBUG);
	return;
}


//***************************************************************************
// Determine total production per month across the map for a given industry type / cargo combination
//***************************************************************************
function MainClass::GetProductionForIndustryTypeCargo(type_id, cargo_id)
{
	local total_production = 0;
	foreach (ind, _ in GSIndustryList()) {
		if (GSIndustry.GetIndustryType(ind) == type_id) {
			total_production += GSIndustry.GetLastMonthProduction(ind, cargo_id);
			Log.Info("Industry: "+GSIndustry.GetName(ind)+" Prod: "+total_production, Log.LVL_DEBUG);
		}
	}
	return total_production;
}

//***************************************************************************
// Will add a goal for a company if not at its limit of 32 (prevented in AddActiveGoal)
//***************************************************************************
function MainClass::CreateChallenge(comp_id)
{
    local cdata = this.companies[comp_id];
	local goal_type = null;
	local cargo_index = null;
	// determine if this goal will be a town growth goal (between 0 and 100%)
	local percentage = GSBase.RandRange(101) - 1;
    if (percentage < GSController.GetSetting("growth_weight")) {
		goal_type = "growth";
	} else {
		goal_type = "delivery";
	}
		
	switch (goal_type) {
		case("growth"): 
			cargo_index = this.SelectCargo(goal_type);
			local cargo = this.cargoes[cargo_index];
			if (cargo_index < 0) break;
			
			local accept = FindGrowthChallenge(cargo_index, comp_id);
			if (accept != null) {
				local population = GSTown.GetPopulation(accept.town);
				local percentage = (GSBase.RandRange(10) + 10).tofloat() / 100;
				local increase = population.tofloat() * percentage;
				if (increase < 100) increase = 100;
				local amount = population + increase.tointeger();
				local dest_name = GSTown.GetName(accept.town);
				if (cdata != null) {
					cdata.AddActiveGoal(cargo, accept, amount, null, goal_type, "minor");
					return;
				}
			}
			break;
	
		case("delivery"):
			for (local attempt = 0;attempt < 20; attempt += 1) {
				cargo_index = this.SelectCargo(goal_type);
				if (cargo_index < 0) continue;
				local cargo = this.cargoes[cargo_index];
				if (cdata.GetNumberOfGoalsForCargo(cargo.cid) > 1) continue; // Already 2 goals for this cargo.
				local distance = GSBase.RandRange(200) + 50; // Distance 50 .. 250 tiles.
				local accept = FindChallenge(cargo_index, distance, comp_id);
				if (accept != null) {
					local amount = GSBase.RandRange(90) + 1;
//					if (amount < 10) {
//						amount = amount * 25; // 25 .. 225
//					} else if (amount < 35) {
//						amount = 10 * 25 + (amount - 10) * 50; // 250..2750
//					} else {
//						amount = 10 * 25 + 35 * 50 + (amount - 35) * 100; // 2000..7500
//					}

					//bob higher cargo requirements as years pass
					local max_years = 0;
					max_years = this.current_year - this.start_year;
					if(max_years > 100)
					{
						max_years = 100;//bob limit increasing cargo sizes
					}
					amount = 300 + 6 * max_years + amount * 25; // 250 .. 2500
//					if ("ind" in accept) {
//						local max_amount = GSIndustry.GetLastMonthProduction(accept.ind, cargo.cid) * 25;
//						if (amount > max_amount) amount = max_amount;
//					}
					if (cdata != null) {
						cdata.AddActiveGoal(cargo, accept, amount, null, goal_type, "minor");
						return;
					}
				}
			}
			break;
		default: break;
	}
}

//***************************************************************************
// Find destinations for the cargo.
// @param cargo_index Cargo index (index in this.cargoes).
// @param comp_id Company to inspect.
// @return A list of destinations, tables 'ind' or 'town' id, and a 'loc' location.
//***************************************************************************
function MainClass::FindGrowthChallenge(cargo_index, comp_id)
{
    local cargo = this.cargoes[cargo_index];
    local dests = {};
	local town_id = null;
    local cdata = this.companies[comp_id];
    local cargo = this.cargoes[cargo_index];

    local acceptable_ratings = [GSTown.TOWN_RATING_MEDIOCRE,    GSTown.TOWN_RATING_GOOD,
								GSTown.TOWN_RATING_VERY_GOOD,   GSTown.TOWN_RATING_EXCELLENT,
								GSTown.TOWN_RATING_OUTSTANDING, GSTown.TOWN_RATING_INVALID];

	// choose destination town at random
	local num_towns = GSTown.GetTownCount() - 1;
	local chosen_iteration = GSBase.RandRange(num_towns);
	local iteration = 0;
	foreach (town, _ in GSTownList()) {
		if (iteration == chosen_iteration) {
			// If the chosen town already has a goal, we won't create this goal, and that's fine
			if (cdata != null && cdata.HasGrowthGoal(town)) return null;
			
			// Otherwise we've found the destination town, but only create this goal if town has an acceptable rating, otherwise forego creating goal
            local rating = GSTown.GetRating(town, comp_id);
			local town_name = GSTown.GetName(town);
			if (rating in acceptable_ratings) {
                local loc = GSTown.GetLocation(town);
                dests[chosen_iteration] <- {town=town, loc=loc};
			} else {
				return null;
			}
		}
		iteration += 1;
	}
	local dest = dests[chosen_iteration];
    return dest;
}

//***************************************************************************
// Select the next cargo to use for a goal.
// @return The index of the cargo to use in this.cargoes.
//***************************************************************************
function MainClass::SelectCargo(goal_type)
{
	local cargo_index = 0;
	switch (goal_type) {
		case("growth"): 
			foreach (cargo in this.cargoes) {
				if (cargo.effect == GSCargo.TE_PASSENGERS) return cargo.index;
			}
		break;
	
		case("delivery"):
			local remaining = GSBase.RandRange(this.sum_weight);
			foreach (cargo in this.cargoes) {
			if (remaining < cargo.weight) return cargo_index;
				remaining -= cargo.weight;
			cargo_index += 1;
		}
		default: break;
	}
    return -1; // Should never be reached.
}

//***************************************************************************
// Select an output cargo for the given industry type 
// @return The index of the cargo to use in this.cargoes.
//***************************************************************************
function MainClass::SelectOutputCargo(type)
{
	local outputcargo = {};
	local num = 0;
	local num_chosen;
	foreach (gamecargo, _ in GSIndustryType.GetProducedCargo(type)) {
		outputcargo[num] <- (gamecargo);
		Log.Info("Output Cargo "+num+ ": "+GSCargo.GetCargoLabel(gamecargo), Log.LVL_DEBUG);
		num +=1;
	}
	if (num == 0) {
		return null;
	} else {
		if (num == 1) {
			num_chosen = 0;
		} else {	
			num_chosen = GSBase.RandRange(num);
		}
	}
	Log.Info("Chosen Output Cargo: "+num_chosen+ ": "+GSCargo.GetCargoLabel(outputcargo[num_chosen]), Log.LVL_DEBUG);

	foreach (cargo in this.cargoes) {
		if (cargo.cid == outputcargo[num_chosen]) return cargo.index;
	}
	return null;
}

//***************************************************************************
// Try to find a challenge for a given cargo and a desired distance.
// @param cargo_index Index in MainClass.cargoes.
// @param distance Desired distance between source and target.
// @param comp_id Company to find a challenge for.
// @return Best accepting industry to use, or 'null' if no industry-pair found.
//***************************************************************************
function MainClass::FindChallenge(cargo_index, distance, comp_id)
{
    local prods = this.FindSources(cargo_index);
    if (prods.len() == 0) return null;
    local accepts = this.FindDestinations(cargo_index, comp_id);
    if (accepts.len() == 0) return null;

    local cdata = this.companies[comp_id];
    local cargo = this.cargoes[cargo_index];

    local best_score = 0; // Best overall distance.
    local best_accept = null; // Best accepting to target.
    foreach (_, accept in accepts) {
        if (cdata != null && cdata.HasGoal(cargo.cid, accept)) continue; // Prevent duplicates.

        local min_prod_distance = distance * 2; // Smallest found distance to the accepting industry.
        local prod_score = best_score;
        foreach (_, prod in prods) {
            local actual_dist = GSTile.GetDistanceManhattanToTile(accept.loc, prod.loc);
            if (actual_dist > distance * 2) continue; // Too far away, skip.

            if (actual_dist < min_prod_distance) {
                min_prod_distance = actual_dist;
                prod_score = this.GetDistanceScore(distance, min_prod_distance);
                if (min_prod_distance < distance && prod_score < best_score) break;
            }
        }
        if (prod_score > best_score) { // The accepting industry is better than what we have.
            prod_score = best_score;
            best_accept = accept;
       }
    }
	return best_accept;
}

//***************************************************************************
// Find cargo sources.
// @param cargo_index Cargo index (index in this.cargoes).
// @return List of resources that produce the requested cargo, list of
//      'ind' or 'town' number, 'prod' produced amount, 'transp' transported amount, and 'loc' location.
//***************************************************************************
function MainClass::FindSources(cargo_index)
{
    local cargo = this.cargoes[cargo_index];
    local num_sources = 0;
    local sources = {};

    if (cargo.freight) {
        // For 'freight' cargoes, check the producing industries for sufficient available production.
        foreach (ind, _ in GSIndustryList_CargoProducing(cargo.cid)) {
            local prod_amount = GSIndustry.GetLastMonthProduction(ind, cargo.cid);
            if (prod_amount < 10) continue;
            local transp_amount = GSIndustry.GetLastMonthTransported(ind, cargo.cid);
            if (prod_amount - transp_amount < 10) continue;
            local loc = GSIndustry.GetLocation(ind);
            sources[num_sources] <- {ind=ind, prod=prod_amount, transp=transp_amount, loc=loc};
            num_sources += 1;
        }
    }
    if (cargo.effect != GSCargo.TE_NONE) {
        // For 'town effect' cargoes, check the towns for sufficient available production.
        foreach (town, _ in GSTownList()) {
            local prod_amount = GSTown.GetLastMonthProduction(town, cargo.cid);
            if (prod_amount < 10) continue;
            local transp_amount = GSTown.GetLastMonthSupplied(town, cargo.cid);
            if (prod_amount - transp_amount < 10) continue;
            local loc = GSTown.GetLocation(town);
            sources[num_sources] <- {town=town, prod=prod_amount, transp=transp_amount, loc=loc};
            num_sources += 1;
        }
    }
// Dump sources for a cargo.
//    GSLog.Info("Sources for " + GSCargo.GetCargoLabel(cargo.cid));
//    foreach (_, src in sources) {
//        if ("ind" in src) {
//            GSLog.Info("Industry " + GSIndustry.GetName(src.ind) + " produces " + src.prod);
//        } else {
//            GSLog.Info("Town " + GSTown.GetName(src.town) + " produces " + src.prod);
//        }
//    }
//    GSLog.Info("");

    return sources;
}

//***************************************************************************
// Find destinations for the cargo.
// @param cargo_index Cargo index (index in this.cargoes).
// @param company Company to inspect.
// @return A list of destinations, tables 'ind' or 'town' id, and a 'loc' location.
//***************************************************************************
function MainClass::FindDestinations(cargo_index, company)
{
    local cargo = this.cargoes[cargo_index];
    local num_dests = 0;
    local dests = {};

    if (cargo.freight) {
        // Assume all industries are willing to accept the cargo.
        foreach (ind, _ in GSIndustryList_CargoAccepting(cargo.cid)) {
            local loc = GSIndustry.GetLocation(ind);
            dests[num_dests] <- {ind=ind, loc=loc};
            num_dests += 1;
        }
    }
    if (cargo.effect != GSCargo.TE_NONE) {
        // Find towns with sufficient rating.
        local acceptable_ratings = [
                GSTown.TOWN_RATING_MEDIOCRE,    GSTown.TOWN_RATING_GOOD,
                GSTown.TOWN_RATING_VERY_GOOD,   GSTown.TOWN_RATING_EXCELLENT,
                GSTown.TOWN_RATING_OUTSTANDING, GSTown.TOWN_RATING_INVALID
        ];

        foreach (town, _ in GSTownList()) {
            local rating = GSTown.GetRating(town, company);
            if (rating in acceptable_ratings) {
                local loc = GSTown.GetLocation(town);
                local amount = GSTile.GetCargoAcceptance(loc, cargo.cid, 1, 1, 4);
                if (amount > 8) {
                    dests[num_dests] <- {town=town, loc=loc};
                    num_dests += 1;
                }
            }
        }
    }
// Dump destinations for a cargo.
//    GSLog.Info("Destinations for " + GSCargo.GetCargoLabel(cargo.cid));
//    foreach (_, dst in dests) {
//        if ("ind" in dst) {
//           GSLog.Info("Industry " + GSIndustry.GetName(dst.ind) + " accepts");
//        } else {
//            GSLog.Info("Town " + GSTown.GetName(dst.town) + " accepts");
//        }
//    }
//    GSLog.Info("");
    return dests;
}


//***************************************************************************
// Construct a score for the distance
// @param desired Desired distance.
// @param actual Actual distance.
// @return Score for the distance.
//***************************************************************************
function MainClass::GetDistanceScore(desired, actual)
{
    if (actual < desired) return 1000 - 3 * (desired - actual); // Too close gets punished hard.
    return 1000 - (actual - desired);
}


//***************************************************************************
// Process events that arrived.
// @return Table with 'force_goal' bool to force goal updating.
//***************************************************************************
function MainClass::ProcessEvents()
{
    local force_goal = false;
    while (GSEventController.IsEventWaiting()) {
        local event = GSEventController.GetNextEvent();
        local event_type = event.GetEventType();

	switch (event.GetEventType()) {
		// On town founding, add a new GoalTown instance
		case GSEvent.ET_TOWN_FOUNDED:
			event = GSEventTownFounded.Convert(event);
			local town_id = event.GetTownID();
			if (GSTown.IsValidTown(town_id)) this.UpdateTownList(town_id);
		break;

        	case GSEvent.ET_COMPANY_NEW:
            		local comp_id = GSEventCompanyNew.Convert(event).GetCompanyID();
            		if (this.companies[comp_id] == null) {
                		this.companies[comp_id] = CompanyData(comp_id);
                		GSLog.Info("Created company " + comp_id + " (newly started).");
                		force_goal = true;
			}
            	break;

        	case GSEvent.ET_COMPANY_MERGER:
            		local comp_id = GSEventCompanyMerger.Convert(event).GetOldCompanyID();
            		if (this.companies[comp_id] != null) {
                		this.companies[comp_id].FinalizeCompany();
                		this.companies[comp_id] = null;
                		GSLog.Info("Deleted company " + comp_id + " (due to merging).");
            		}
		break;

		case GSEvent.ET_COMPANY_BANKRUPT:
		        local comp_id = GSEventCompanyBankrupt.Convert(event).GetCompanyID();
            		if (this.companies[comp_id] != null) {
                		this.companies[comp_id].FinalizeCompany();
                		this.companies[comp_id] = null;
                		GSLog.Info("Deleted company " + comp_id + " (no monies anymore).");
            		}
		break;

        	case GSEvent.ET_INDUSTRY_CLOSE:
            		local ind_id = GSEventIndustryClose.Convert(event).GetIndustryID();
            		foreach (comp_id, cdata in this.companies) {
                		if (cdata != null) cdata.IndustryClosed(ind_id);
            		}
		break;

		default: break;
        }
    }
    return {force_goal=force_goal};
}

//***************************************************************************
//***************************************************************************
function MainClass::Save()
{
	Log.Info("Saving data...", Log.LVL_INFO);
	local save_table = {};

	// If some permanent setting has been changed in scenario editor, do not save anything.
	if ((::SettingsTable.industry_NewGRF != GSController.GetSetting("industry_NewGRF")) ||
	    (::SettingsTable.use_town_sign != GSController.GetSetting("use_town_sign"))) {
		Log.Info("Some permanent setting changed. Not saving town data.", Log.LVL_INFO);
		return save_table;
	}

	// If the script isn't yet initialized, we can't retrieve data from GoalTown instances. Thus, simply use the original
	// loaded table. Otherwise we build the table with town data.
	
	save_table.town_data_table <- {};
	save_table.company_data_table <- {};
	if (!this.gs_init_done) {
		save_table.town_data_table <- ::TownDataTable;
       	save_table.company_data_table <- ::CompanyDataTable;
	} 
	else {
		// save towns
		foreach (i, town in this.towns) {
			save_table.town_data_table[town.id] <- town.SavingTownData();
		}
		// save companies
		foreach (comp_id, cdata in this.companies) {
        		if (cdata == null) continue;
        		save_table.company_data_table[comp_id] <- cdata.SaveCompany();
    	}
		// Also store a savegame version flag and the running goal totals
		save_table.save_version <- this.current_save_version;
		save_table.goals_completed_total <- this.goals_completed_total;
		save_table.goals_expired_total <- this.goals_expired_total;
		save_table.goals_total_id <- this.goals_total_id;
		
		save_table.start_year <- this.start_year;//bob
		save_table.terraform_completion <- this.terraform_completion;
	}
	return save_table;
}

//***************************************************************************
//***************************************************************************
function MainClass::Load(version, saved_data)
{
	Log.Info("Loading data...", Log.LVL_INFO);

    // Examine and store cargo types of the game.
    this.cargoes = {};
    this.num_cargoes = 0;
    this.sum_weight = 0;

    for (local cid = 0; cid < 32; cid += 1) {
        if (!GSCargo.IsValidCargo(cid)) continue;

        local cargo = Cargo(this.num_cargoes, cid, GSCargo.IsFreight(cid),  GSCargo.GetTownEffect(cid));
        this.cargoes[this.num_cargoes] <- cargo;
        this.num_cargoes += 1;
        this.sum_weight += cargo.weight;
    }

    // Construct empty companies.
    this.companies = {};
    for (local comp_id = GSCompany.COMPANY_FIRST; comp_id <= GSCompany.COMPANY_LAST; comp_id++) {
        this.companies[comp_id] <- null;
    }

	// load data if the savegame version matches.
	if (saved_data.rawin("save_version") && saved_data.save_version == this.current_save_version) {
		this.load_saved_data = true;
		this.goals_completed_total = saved_data.goals_completed_total;
		this.goals_expired_total = saved_data.goals_expired_total;
		this.goals_total_id = saved_data.goals_total_id;
				
		this.start_year = saved_data.start_year;//bob
		this.terraform_completion = saved_data.terraform_completion;//bob
		
		foreach (townid, town_data in saved_data.town_data_table) {
			::TownDataTable[townid] <- town_data;
		}
		foreach (comp_id, loaded_comp_data in saved_data.company_data_table) {
//			::CompanyDataTable[comp_id] <- loaded_comp_data;
        		local cdata = CompanyData.LoadCompany(comp_id, loaded_comp_data, this.cargoes);
        		this.companies[comp_id] = cdata;
		}
	}
	else {
		Log.Info("Data format doesn't match with current version. Resetting.", Log.LVL_INFO);
	}
}

//***************************************************************************
// The script data got loaded from file, check it against the game.
//***************************************************************************
function MainClass::CompaniesPostLoadCheck()
{
    // Check companies.
    for (local comp_id = GSCompany.COMPANY_FIRST; comp_id <= GSCompany.COMPANY_LAST; comp_id++) {
        if (GSCompany.ResolveCompanyID(comp_id) == GSCompany.COMPANY_INVALID) {
            if (this.companies[comp_id] != null) {
                this.companies[comp_id].FinalizeCompany();
                this.companies[comp_id] = null;
                GSLog.Info("Deleted company " + comp_id + " (disappeared after loading).");
            }
        } else {
            if (this.companies[comp_id] == null) {
                this.companies[comp_id] = CompanyData(comp_id);
                GSLog.Info("Created company " + comp_id + " (appeared from nowhere after loading).");
            }
        }
    }

    // Check industries used for goals.
    foreach (comp_id, cdata in this.companies) {
        if (cdata == null) continue;
        cdata.GoalsPostLoadCheck();
    }
}

